"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const npmlog_1 = __importDefault(require("npmlog"));
const taskId_1 = require("../task/taskId");
const stream_1 = require("stream");
const chalk_1 = __importDefault(require("chalk"));
const taskLogs = new Map();
const maxLengths = {
    pkg: 0,
    task: 0,
};
const colors = {
    info: chalk_1.default.white,
    verbose: chalk_1.default.gray,
    warn: chalk_1.default.white,
    error: chalk_1.default.white,
    task: chalk_1.default.cyan,
    pkg: chalk_1.default.magenta,
};
function getTaskLogs() {
    return taskLogs;
}
exports.getTaskLogs = getTaskLogs;
function setTaskLogMaxLengths(maxPkgLength, maxTaskLength) {
    maxLengths.pkg = maxPkgLength;
    maxLengths.task = maxTaskLength;
}
exports.setTaskLogMaxLengths = setTaskLogMaxLengths;
function getTaskLogPrefix(pkg, task) {
    return `${colors.pkg(pkg.padStart(maxLengths.pkg))} ${colors.task(task.padStart(maxLengths.task))}`;
}
exports.getTaskLogPrefix = getTaskLogPrefix;
function addToTaskLog(pkg, task, message) {
    const taskId = taskId_1.getTaskId(pkg, task);
    if (!taskLogs.has(taskId)) {
        taskLogs.set(taskId, []);
    }
    taskLogs.get(taskId).push(message);
}
function normalize(prefixOrMessage, message) {
    if (typeof message === "string") {
        const prefix = prefixOrMessage;
        return { prefix, message };
    }
    else {
        const prefix = "";
        const message = prefixOrMessage;
        return { prefix, message };
    }
}
function info(prefixOrMessage, message) {
    const normalizedArgs = normalize(prefixOrMessage, message);
    return npmlog_1.default.info(normalizedArgs.prefix, colors.info(normalizedArgs.message));
}
function warn(prefixOrMessage, message) {
    const normalizedArgs = normalize(prefixOrMessage, message);
    return npmlog_1.default.warn(normalizedArgs.prefix, colors.warn(normalizedArgs.message));
}
function error(prefixOrMessage, message) {
    const normalizedArgs = normalize(prefixOrMessage, message);
    return npmlog_1.default.error(normalizedArgs.prefix, colors.error(normalizedArgs.message));
}
function verbose(prefixOrMessage, message, ...args) {
    const normalizedArgs = normalize(prefixOrMessage, message);
    return npmlog_1.default.verbose(normalizedArgs.prefix, colors.verbose(normalizedArgs.message));
}
class NpmLogWritable extends stream_1.Writable {
    constructor(pkg, task) {
        super();
        this.buffer = "";
        this.taskLogger = taskLogger(pkg, task);
    }
    _write(chunk, _encoding, callback) {
        let prev = 0;
        let curr = 0;
        while (curr < chunk.byteLength) {
            if (chunk[curr] === 13 || (chunk[curr] === 10 && curr - prev > 1)) {
                this.buffer =
                    this.buffer +
                        chunk
                            .slice(prev, curr)
                            .toString()
                            .trim();
                this.taskLogger.verbose(this.buffer);
                this.buffer = "";
                prev = curr;
            }
            curr++;
        }
        callback();
    }
}
exports.NpmLogWritable = NpmLogWritable;
function logLevel(level) {
    npmlog_1.default.level = level;
}
exports.logLevel = logLevel;
exports.logger = {
    info,
    warn,
    error,
    verbose,
};
function taskLogger(pkg, task) {
    return {
        info: (message) => {
            addToTaskLog(pkg, task, message);
            return npmlog_1.default.info(getTaskLogPrefix(pkg, task), colors.info(message));
        },
        warn: (message) => {
            addToTaskLog(pkg, task, message);
            return npmlog_1.default.warns(getTaskLogPrefix(pkg, task), colors.warn(message));
        },
        error: (message) => {
            addToTaskLog(pkg, task, message);
            return npmlog_1.default.error(getTaskLogPrefix(pkg, task), colors.error(message));
        },
        verbose: (message) => {
            addToTaskLog(pkg, task, message);
            return npmlog_1.default.verbose(getTaskLogPrefix(pkg, task), colors.verbose(message));
        },
    };
}
exports.taskLogger = taskLogger;
